
import { _decorator, color, Color, Component, director, gfx, Node, Texture2D, v2, Vec2 } from 'cc';
const { ccclass, property } = _decorator;

@ccclass('Test')
export class Test extends Component {

    @property(Texture2D)
    texture: Texture2D = null!;

    sourceData: Uint8Array = null!;

    start() {
        this.readPixels();
        const numberOfEnclosedShapes = this.countEnclosedShapes(this.texture.width, this.texture.height);
        console.log("能够形成的包围图形数量:", numberOfEnclosedShapes);
    }

    readPixels() {
        this.sourceData = this.readTexturePixels(this.texture);
    }

    countEnclosedShapes(width: number, height: number): number {
        const grid: Uint8Array = Uint8Array.from(this.sourceData);
        const visited = new Uint8Array(grid.length / 4); // 记录每个像素是否已被访问

        // 定义迭代的洪水填充算法
        const floodFill = (x: number, y: number): void => {
            const stack: { x: number, y: number }[] = [{ x, y }];

            while (stack.length > 0) {
                const { x, y } = stack.pop()!;

                // 计算当前像素在一维数组中的索引
                const index = y * width + x;

                // 检查边界条件和透明度
                if (x < 0 || x >= width || y < 0 || y >= height ||
                    visited[index] || (grid[index * 4 + 3] === 0)) { // 检查透明度（A）
                    continue;
                }

                visited[index] = 1; // 标记为已访问

                // 将四个邻接的点加入栈
                stack.push({ x: x + 1, y });
                stack.push({ x: x - 1, y });
                stack.push({ x, y: y + 1 });
                stack.push({ x, y: y - 1 });
            }
        }

        let shapeCount = 0;

        // 遍历像素网格进行填充
        for (let y = 0; y < height; y++) {
            for (let x = 0; x < width; x++) {
                const index = y * width + x;
                // 检查目前的像素是否未被访问和透明度不为0
                if (!visited[index] && grid[index * 4 + 3] !== 0) { // 透明度不为0
                    floodFill(x, y); // 从这个点开始填充
                    shapeCount++; // 成功填充一个区域，计数
                }
            }
        }

        return shapeCount;
    }

    /**
     * 读取texture的像素到一个buffer
     * @param texture texture原图
     * @param width 
     * @param height 
     * @param x 
     * @param y 
     * @returns 
     */
    public readTexturePixels(texture: Texture2D, x = 0, y = 0): Uint8Array {
        const gfxTexture = texture.getGFXTexture();
        if (gfxTexture == null) return null;

        const bufferSize = 4 * texture.width * texture.height;
        const buffer = new Uint8Array(bufferSize);

        const region = new gfx.BufferTextureCopy();
        region.texOffset.x = x;
        region.texOffset.y = y;
        region.texExtent.width = texture.width;
        region.texExtent.height = texture.height;

        director.root.device.copyTextureToBuffers(gfxTexture, [buffer], [region]);
        return buffer;
    }
}
